@@ -4,7 +4,7 @@ __copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
 __license__ = 'GPL 2.0/LGPL 2.1'
 __all__ = ('Connection',)
 
-from starbase.exceptions import ImproperlyConfigured
+from starbase.exceptions import ImproperlyConfigured, DoesNotExist
 from starbase.content_types import CONTENT_TYPES_DICT, CONTENT_TYPES, DEFAULT_CONTENT_TYPE
 from starbase.defaults import HOST, PORT, USER, PASSWORD, PERFECT_DICT
 from starbase.client.table import Table
@@ -58,40 +58,41 @@ class Connection(object):
             'secure': 's' if self.secure else '',
             'host': self.host,
             'port': self.port,
-            #'user_credentials': ("{0}:{1}@".format(self.user, self.password)) if (self.user and self.password) else ''
         }
-        #self.base_url = 'http{secure}://{user_credentials}{host}:{port}/'.format(**data)
         self.base_url = 'http{secure}://{host}:{port}/'.format(**data)
 
     @property
-    def version(self):
+    def version(self, fail_silently=True):
         """
         Software version. Returns the software version.
 
+        :param bool fail_silently:
         :return dict: Dictionary with info on software versions (OS, Server, JVM, etc).
         """
-        response = HttpRequest(connection=self, url='version').get_response()
+        response = HttpRequest(connection=self, url='version', fail_silently=fail_silently).get_response()
         return response.content
 
     @property
-    def cluster_version(self):
+    def cluster_version(self, fail_silently=True):
         """
         Storage cluster version. Returns version information regarding the HBase cluster backing the Stargate
         instance.
 
+        :param bool fail_silently:
         :return str: HBase version.
         """
-        response = HttpRequest(connection=self, url='version/cluster').get_response()
+        response = HttpRequest(connection=self, url='version/cluster', fail_silently=fail_silently).get_response()
         return response.content
 
     @property
-    def cluster_status(self):
+    def cluster_status(self, fail_silently=True):
         """
         Storage cluster satus. Returns detailed status on the HBase cluster backing the Stargate instance.
 
+        :param bool fail_silently:
         :return dict: Dictionary with information on dead nodes, live nodes, average load, regions, etc.
         """
-        response = HttpRequest(connection=self, url='status/cluster').get_response()
+        response = HttpRequest(connection=self, url='status/cluster', fail_silently=fail_silently).get_response()
         return response.content
 
     def table(self, name):
@@ -101,23 +102,23 @@ class Connection(object):
         :param str name: Table name. Example value 'test'.
         :return stargate.base.Table:
 
-        .. note::
-            This method does not check if table exists. Use the following methods to perform the
-            check:
+        This method does not check if table exists. Use the following methods to perform the
+        check:
 
-                - `starbase.client.Connection.table_exists` or
-                - `starbase.client.table.Table.exists`.
+            - `starbase.client.Connection.table_exists` or
+            - `starbase.client.table.Table.exists`.
         """
         return Table(connection=self, name=name)
 
-    def tables(self, raw=False):
+    def tables(self, raw=False, fail_silently=True):
         """
         Table list. Retrieves the list of available tables.
 
         :param bool raw: If set to True raw result (JSON) is returned.
+        :param bool fail_silently:
         :return list: Just a list of plain strings of table names, no Table instances.
         """
-        response = HttpRequest(connection=self).get_response()
+        response = HttpRequest(connection=self, fail_silently=fail_silently).get_response()
         if not raw:
             try:
                 return [table['name'] for table in response.content['table']]
@@ -125,14 +126,20 @@ class Connection(object):
                 return []
         return response.content
 
-    def table_exists(self, name):
+    def table_exists(self, name, fail_silently=True):
         """
         Checks if table exists.
 
         :param str name: Table name.
+        :param bool fail_silently:
         :return bool:
         """
-        return name in self.tables()
+        table_exists = name in self.tables()
+
+        if not table_exists and not fail_silently:
+            raise DoesNotExist("Table `{0}` does not exist!".format(name))
+
+        return table_exists
 
     # ******************** Helper methods ********************
     def create_table(self, name, *columns):
@@ -149,7 +156,7 @@ class Connection(object):
             table.create(*columns)
             return table
 
-    def drop_table(self, name):
+    def drop_table(self, name, fail_silently=True):
         """
         Drops the table.
 
@@ -157,4 +164,4 @@ class Connection(object):
         :return int: Status code.
         """
         table = self.table(name)
-        return table.drop()
+        return table.drop(fail_silently=fail_silently)
