@@ -7,8 +7,13 @@ import threading
 import multiprocessing
 import unittest
 import uuid
+import shutil
+import os
+import binascii
 
-from six import text_type, PY3, print_
+PROJECT_DIR = lambda base : os.path.abspath(os.path.join(os.path.dirname(__file__), base).replace('\\','/'))
+
+from six import text_type, PY3, print_, BytesIO as StringIO
 
 try:
     from six.moves import range as xrange
@@ -16,11 +21,23 @@ except ImportError:
     if PY3:
         xrange = range
 
+try:
+    from six.moves.urllib.request import build_opener
+except ImportError:
+    if PY3:
+        from urllib.request import build_opener
+    else:
+        from urllib2 import build_opener
+
+from requests.exceptions import HTTPError
+
 from starbase import Connection, Table
+from starbase.exceptions import DoesNotExist, ParseError
 
 HOST = '127.0.0.1'
 PORT = 8000
 TABLE_NAME = 'messages'
+NON_EXISTENT_TABLE_NAME = 'user_stats'
 
 COLUMN_FROM_USER = 'from_user'
 FIELD_FROM_USER_ID = 'id'
@@ -400,14 +417,8 @@ class StarbaseClient02TableTest(unittest.TestCase):
         """
         return self.test_06_table_batch_post_multiple_column_data(process_number=process_number, perfect_dict=True)
 
-    @print_info
-    def test_08_table_put_column_data(self, process_number=0):
-        """
-        Insert single column data into a single row of HBase using starbase REST API.
-        """
-        key = 'row_1_'
+    def __table_put_column_data_2(self, key, num_rows):
         res = []
-        num_rows = NUM_ROWS
 
         for i in xrange(num_rows):
             columns = {
@@ -422,6 +433,18 @@ class StarbaseClient02TableTest(unittest.TestCase):
                 }
             res.append(self.table.insert('{0}{1}'.format(key, i), columns))
 
+        return res
+
+    @print_info
+    def test_08_table_put_column_data(self, process_number=0):
+        """
+        Insert single column data into a single row of HBase using starbase REST API.
+        """
+        key = 'row_1_'
+        num_rows = NUM_ROWS
+
+        res = self.__table_put_column_data_2(key, num_rows)
+
         self.assertEqual(res, [200 for i in xrange(num_rows)])
         return res
 
@@ -448,15 +471,7 @@ class StarbaseClient02TableTest(unittest.TestCase):
         self.assertEqual(res, 200)
         return res
 
-    @print_info
-    def test_10_table_put_column_data(self, process_number=0):
-        """
-        Insert multiple column data into a single row of HBase using starbase REST API.
-
-        ..note: Used in ``test_11_get_single_row_with_all_columns`` and ``test_08b_get_single_row_with_all_columns``.
-        """
-        key = 'row_2_abcdef'
-
+    def __table_put_column_data(self, key='row_2_abcdef'):
         columns = {
             '{0}:{1}'.format(COLUMN_FROM_USER, FIELD_FROM_USER_ID): '110',
             '{0}:{1}'.format(COLUMN_FROM_USER, FIELD_FROM_USER_NAME): 'John Doe',
@@ -468,14 +483,28 @@ class StarbaseClient02TableTest(unittest.TestCase):
             '{0}:{1}'.format(COLUMN_MESSAGE, FIELD_MESSAGE_BODY): 'Lorem ipsum dolor sit amet.',
             }
         res = self.table.insert(key, columns)
+        return res
+
+    @print_info
+    def test_10_table_put_column_data(self, process_number=0):
+        """
+        Insert multiple column data into a single row of HBase using starbase REST API.
+
+        ..note: Used in ``test_11_get_single_row_with_all_columns`` and ``test_08b_get_single_row_with_all_columns``.
+        """
+        key = 'row_2_abcdef'
+
+        res = self.__table_put_column_data(key)
         self.assertEqual(res, 200)
         return res
 
     @print_info
-    def test_11_get_single_row_with_all_columns(self, row_key='row_2_abcdef'):
+    def test_11_get_single_row_with_all_columns(self, row_key='row_2_abcdef__11'):
         """
         Fetches a single row from HBase using starbase REST API with all columns of that row as simple dict.
         """
+        self.__table_put_column_data(row_key)
+
         res = self.table.fetch(row=row_key, perfect_dict=False)
         output = {
             'from_user:id': '110',
@@ -491,10 +520,12 @@ class StarbaseClient02TableTest(unittest.TestCase):
         return res
 
     @print_info
-    def test_16_get_single_row_with_all_columns_as_perfect_dict(self, row_key='row_2_abcdef'):
+    def test_16_get_single_row_with_all_columns_as_perfect_dict(self, row_key='row_2_abcdef__16'):
         """
         Fetches a single row from HBase using starbase REST API with all columns of that row as perfect dict.
         """
+        self.__table_put_column_data(row_key)
+
         res = self.table.fetch(row=row_key, perfect_dict=True)
         output = {
             'to_user': {'id': '220', 'name': 'Lorem Ipsum', 'email': 'lorem@ipsum.net'},
@@ -510,6 +541,8 @@ class StarbaseClient02TableTest(unittest.TestCase):
         Updates (POST) data of a single row of HBase using starbase REST API. Updates data set in
         ``test_09_table_put_column_data``.
         """
+        # TODO: This is not a well done test.
+
         key = 'row_1_abcdef'
 
         columns = {
@@ -543,10 +576,12 @@ class StarbaseClient02TableTest(unittest.TestCase):
         return res
 
     @print_info
-    def test_14_get_single_row_with_all_columns(self, row_key='row_1_abcdef'):
+    def test_14_get_single_row_with_all_columns(self, row_key='row_1_abcdef__14'):
         """
         Fetches a single row from HBase using starbase REST API with all columns of that row.
         """
+        self.__table_put_column_data(row_key)
+
         res = self.table.fetch(row=row_key, perfect_dict=True)
         output = {
             'to_user': {'id': '220', 'email': 'lorem@ipsum.net', 'name': 'Lorem Ipsum'},
@@ -563,14 +598,17 @@ class StarbaseClient02TableTest(unittest.TestCase):
         Insert single column data into a single row of HBase using starbase REST API. Deletes data set by
         ``test_08_table_put_column_data`` (all except the last record)..
         """
-        key = 'row_1_'
+        key = 'row_1_15_'
         res = []
         num_rows = NUM_ROWS - 1
+
+        res2 = self.__table_put_column_data_2(key, num_rows)
+
         output = []
         for i in xrange(num_rows):
-            columns = {
-                '{0}:{1}'.format(COLUMN_FROM_USER, FIELD_FROM_USER_ID): str(11 * (i + 1)),
-                }
+            #columns = {
+            #    '{0}:{1}'.format(COLUMN_FROM_USER, FIELD_FROM_USER_ID): str(11 * (i + 1)),
+            #    }
             res.append(self.table.remove('{0}{1}'.format(key, i)))
             output.append(200)
 
@@ -594,10 +632,14 @@ class StarbaseClient02TableTest(unittest.TestCase):
         return res
 
     @print_info
-    def test_17_get_single_row_with_selective_columns(self, row_key='row_1_9'):
+    def test_17_get_single_row_with_selective_columns(self, row_key='row_1_9_17'):
         """
         Fetches a single row selective columns from HBase using starbase REST API.
         """
+        # TODO: This is not a well done test.
+
+        self.__table_put_column_data(row_key)
+
         # Columns to fetch (normal list)
         columns = [
             '{0}:{1}'.format(COLUMN_FROM_USER, FIELD_FROM_USER_ID),
@@ -613,7 +655,7 @@ class StarbaseClient02TableTest(unittest.TestCase):
         ]
 
         # Get table row data
-        res = self.table.fetch(row=TEST_ROW_KEY_1, columns=columns, perfect_dict=True)
+        res = self.table.fetch(row=row_key, columns=columns, perfect_dict=True)
 
         return res
 
@@ -664,9 +706,14 @@ class StarbaseClient02TableTest(unittest.TestCase):
         """
         Get all rows.
         """
+        data1 = {'from_user': {'id': 'ku', 'name': 'tra'}, 'to_user': {'order': '2', 'she': '1'}}
+        self.table.insert('papa', data1)
+        data2 = {'from_user': {'id': 'zu', 'name': 'za'}, 'to_user': {'genius': 'yep', 'she': 'likes'}}
+        self.table.insert('mama', data2)
+
         res = list(self.table.fetch_all_rows(perfect_dict=perfect_dict))
-        self.assertEqual(res[0]['to_user'], {'id': '220', 'email': 'lorem@ipsum.net', 'name': 'Lorem Ipsum'})
-        self.assertEqual(res[1]['from_user'], {'id': '110', 'name': 'John Doe', 'email': 'john@doe.net'})
+        self.assertEqual(res[0]['to_user'], data2['to_user'])
+        self.assertEqual(res[1]['from_user'], data1['from_user'])
         return res
 
     @print_info
@@ -674,19 +721,23 @@ class StarbaseClient02TableTest(unittest.TestCase):
         """
         Get all rows with filter string
         """
-        row_filter_string = '{"type": "RowFilter", "op": "EQUAL", "comparator": {"type": "RegexStringComparator", "value": "^row_1.+" }}'
+        data = {
+            'row_1_9': {'to_user': {'email': 'lorem@ipsum.net', 'name': 'Lorem Ipsum', 'id': '220'},
+            'message': {'body': 'Lorem ipsum dolor sit amet.', 'subject': 'Lorem ipsum'},
+            'from_user': {'email': 'john@doe.net', 'name': 'John Doe', 'id': '110'}}
+        }
+
+        key_prefix = 'pow_1'
+
+        for i in xrange(20):
+            self.table.insert('{0}_{1}'.format(key_prefix, i), data)
+
+        row_filter_string = '{{"type": "RowFilter", "op": "EQUAL", "comparator": {{"type": "RegexStringComparator", "value": "^{0}.+" }}}}'.format(key_prefix)
 
         res = list(self.table.fetch_all_rows(with_row_id=True, perfect_dict=perfect_dict, filter_string=row_filter_string))
 
         for row in res:
-            self.assertEqual(
-                row,
-                {
-                    'row_1_9': {'to_user': {'email': 'lorem@ipsum.net', 'name': 'Lorem Ipsum', 'id': '220'},
-                    'message': {'body': 'Lorem ipsum dolor sit amet.', 'subject': 'Lorem ipsum'},
-                    'from_user': {'email': 'john@doe.net', 'name': 'John Doe', 'id': '110'}}
-                }
-                )
+            self.assertEqual(row, data)
             break
 
         return res
@@ -696,19 +747,23 @@ class StarbaseClient02TableTest(unittest.TestCase):
         """
         Get all rows with scanner config
         """
-        scanner_config = '<Scanner maxVersions="1"><filter>{"op":"EQUAL", "type":"RowFilter", "comparator":{"value":"^row_1.+","type":"RegexStringComparator"}}</filter></Scanner>'
+        data = {
+            'row_1_9_19': {'to_user': {'email': 'lorem@ipsum.net', 'name': 'Lorem Ipsum', 'id': '220'},
+            'message': {'body': 'Lorem ipsum dolor sit amet.', 'subject': 'Lorem ipsum'},
+            'from_user': {'email': 'john@doe.net', 'name': 'John Doe', 'id': '110'}}
+        }
+
+        key_prefix = 'bow_1'
+
+        for i in xrange(20):
+            self.table.insert('{0}_{1}'.format(key_prefix, i), data)
+
+        scanner_config = '<Scanner maxVersions="1"><filter>{{"op":"EQUAL", "type":"RowFilter", "comparator":{{"value":"^{0}.+","type":"RegexStringComparator"}}}}</filter></Scanner>'.format(key_prefix)
 
         res = list(self.table.fetch_all_rows(with_row_id=True, perfect_dict=perfect_dict, scanner_config=scanner_config))
 
         for row in res:
-            self.assertEqual(
-                row,
-                {
-                    'row_1_9': {'to_user': {'email': 'lorem@ipsum.net', 'name': 'Lorem Ipsum', 'id': '220'},
-                    'message': {'body': 'Lorem ipsum dolor sit amet.', 'subject': 'Lorem ipsum'},
-                    'from_user': {'email': 'john@doe.net', 'name': 'John Doe', 'id': '110'}}
-                }
-                )
+            self.assertEqual(row, data)
             break
 
         return res
@@ -989,6 +1044,215 @@ class StarbaseClient02TableTest(unittest.TestCase):
 
         return (r1, r2, r3, r4)
 
+    def __insert_binary_file(self, url):
+        """
+        Insert a binary file. First download the file and then insert.
+        """
+        opener = build_opener()
+        page = opener.open(url)
+        image = binascii.b2a_hex(page.read())
+        return image.decode()
+
+    @print_info
+    def test_25_insert_binary_file(self):
+        """
+        Store binary file.
+        """
+        # Write binary file into HBase
+        url = 'https://raw.github.com/barseghyanartur/delusionalinsanity.images/master/images/32013_394119419025_539104025_3916154_3598710_n.jpg'
+        image = self.__insert_binary_file(url)
+
+        data = {
+            COLUMN_MESSAGE: {'text': 'John', 'new': 'yes', 'image': image},
+            COLUMN_FROM_USER: {'id': '555', 'email': 'fr@m.com'},
+        }
+
+        row_key = 'image_test_1'
+        write_res = self.table.insert(row_key, data)
+
+        self.assertEqual(write_res, 200)
+
+        # Get file from HBase and compare source
+        read_res = self.table.fetch(row_key, COLUMN_MESSAGE, ['image'])
+
+        self.assertEqual(read_res[COLUMN_MESSAGE]['image'], image)
+
+        f = open('file.jpg', 'wb')
+        f.write(binascii.a2b_hex(read_res[COLUMN_MESSAGE]['image']))
+
+    def __insert_row_into_non_existing_table(self, fail_silently=True):
+        """
+        Insert row into non-existing table.
+        """
+        # Success test
+        perfect_dict = True
+
+        key = 'row_{0}_{1}'.format(('perfect_' if perfect_dict else ''), str(uuid.uuid4()))
+
+        columns = {
+            COLUMN_FROM_USER: {
+                FIELD_FROM_USER_ID: '123',
+                FIELD_FROM_USER_NAME: 'John Doe',
+                FIELD_FROM_USER_EMAIL: 'john@doe.com'
+            },
+            COLUMN_TO_USER: {
+                FIELD_TO_USER_ID: '456',
+                FIELD_TO_USER_NAME: 'Lorem Ipsum',
+                FIELD_TO_USER_EMAIL: 'lorem@ipsum.com'
+            },
+            COLUMN_MESSAGE: {
+                FIELD_MESSAGE_SUBJECT: 'Lorem ipsum',
+                FIELD_MESSAGE_BODY: 'Lorem ipsum dolor sit amet.'
+            },
+        }
+        table = self.connection.table(NON_EXISTENT_TABLE_NAME)
+        res = table.insert(key, columns, fail_silently=fail_silently)
+        return res
+
+    @print_info
+    def test_26_insert_row_into_non_existing_table_fail_silently(self):
+        """
+        Insert row into non-existing table (`fail_silently` set to True).
+        """
+        res = self.__insert_row_into_non_existing_table(fail_silently=True)
+        self.assertEqual(res, None)
+
+    @print_info
+    def test_27_insert_row_into_non_existing_table_raise_exception(self):
+        """
+        Insert row into non-existing table (`fail_silently` set to False).
+        """
+        try:
+            res = self.__insert_row_into_non_existing_table(fail_silently=False)
+            raise Exception("`starbase.exceptions.DoesNotExist` is expected to be raised, but it's not!")
+        except DoesNotExist as e:
+            pass
+
+    def __update_row_of_non_existing_table(self, fail_silently=True):
+        """
+        Update row of non-existing table.
+        """
+        # Success test
+        perfect_dict = True
+
+        key = 'row_{0}_{1}'.format(('perfect_' if perfect_dict else ''), str(uuid.uuid4()))
+
+        columns = {
+            COLUMN_FROM_USER: {
+                FIELD_FROM_USER_ID: '123',
+                FIELD_FROM_USER_NAME: 'John Doe',
+                FIELD_FROM_USER_EMAIL: 'john@doe.com'
+            },
+            COLUMN_TO_USER: {
+                FIELD_TO_USER_ID: '456',
+                FIELD_TO_USER_NAME: 'Lorem Ipsum',
+                FIELD_TO_USER_EMAIL: 'lorem@ipsum.com'
+            },
+            COLUMN_MESSAGE: {
+                FIELD_MESSAGE_SUBJECT: 'Lorem ipsum',
+                FIELD_MESSAGE_BODY: 'Lorem ipsum dolor sit amet.'
+            },
+        }
+        table = self.connection.table(NON_EXISTENT_TABLE_NAME)
+        res = table.update(key, columns, fail_silently=fail_silently)
+        return res
+
+    @print_info
+    def test_28_update_row_of_non_existing_table_fail_silently(self):
+        """
+        Update row of non-existing table (`fail_silently` set to True).
+        """
+        res = self.__update_row_of_non_existing_table(fail_silently=True)
+        self.assertEqual(res, None)
+
+    @print_info
+    def test_29_update_row_of_non_existing_table_raise_exception(self):
+        """
+        Update row of non-existing table (`fail_silently` set to False).
+        """
+        try:
+            res = self.__update_row_of_non_existing_table(fail_silently=False)
+            raise Exception("`starbase.exceptions.DoesNotExist` is expected to be raised, but it's not!")
+        except DoesNotExist as e:
+            pass
+
+    def __drop_non_existing_table_fail_silently(self, fail_silently=True):
+        """
+        Drop non-existing table.
+        """
+        table = self.connection.table(NON_EXISTENT_TABLE_NAME)
+        return table.drop(fail_silently=fail_silently)
+
+    @print_info
+    def test_30_drop_non_existing_table_fail_silently(self):
+        """
+        Drop non-existing table (`fail_silently` set to True).
+        """
+        res = self.__drop_non_existing_table_fail_silently(fail_silently=True)
+        self.assertEqual(res, 503)
+
+    @print_info
+    def test_31_drop_non_existing_table_raise_exception(self):
+        """
+        Drop non-existing table  (`fail_silently` set to False).
+        """
+        try:
+            res = self.__drop_non_existing_table_fail_silently(fail_silently=False)
+            raise Exception("`requests.exceptions.HTTPError` is expected to be raised, but it's not!")
+        except HTTPError as e:
+            pass
+
+    def __fetch_row_of_non_existing_table(self, fail_silently=True):
+        """
+        Fetch row of non existing table.
+        """
+        table = self.connection.table(NON_EXISTENT_TABLE_NAME)
+        return table.fetch('bla_01', fail_silently=fail_silently)
+
+    @print_info
+    def test_32_fetch_row_of_non_existing_table_fail_silently(self):
+        """
+        Drop non-existing table (`fail_silently` set to True).
+        """
+        res = self.__fetch_row_of_non_existing_table(fail_silently=True)
+        self.assertEqual(res, None)
+
+    @print_info
+    def test_33_fetch_row_of_non_existing_table_raise_exception(self):
+        """
+        Drop non-existing table  (`fail_silently` set to False).
+        """
+        try:
+            res = self.__fetch_row_of_non_existing_table(fail_silently=False)
+            raise Exception("`starbase.exceptions.DoesNotExist` is expected to be raised, but it's not!")
+        except DoesNotExist as e:
+            pass
+
+    def __remove_row_of_non_existing_table(self, fail_silently=True):
+        """
+        Remove row of non existing table.
+        """
+        table = self.connection.table(NON_EXISTENT_TABLE_NAME)
+        return table.remove('bla_01', fail_silently=fail_silently)
+
+    @print_info
+    def test_34_remove_row_of_non_existing_table_fail_silently(self):
+        """
+        Remove row of non-existing table (`fail_silently` set to True).
+        """
+        res = self.__remove_row_of_non_existing_table(fail_silently=True)
+        self.assertEqual(res, 500)
+
+    @print_info
+    def test_35_remove_row_of_non_existing_table_raise_exception(self):
+        """
+        Remove row of non-existing table  (`fail_silently` set to False).
+        """
+        try:
+            res = self.__remove_row_of_non_existing_table(fail_silently=False)
+            raise Exception("`starbase.exceptions.DoesNotExist` is expected to be raised, but it's not!")
+        except HTTPError as e:
+            pass
 
 if __name__ == '__main__':
     unittest.main()
